# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .ingest_event_param import IngestEventParam

__all__ = ["IngestBulkParams"]


class IngestBulkParams(TypedDict, total=False):
    events: Required[Iterable[IngestEventParam]]


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .experience_type import ExperienceType as ExperienceType
from .type_list_params import TypeListParams as TypeListParams
from .type_create_params import TypeCreateParams as TypeCreateParams
from .type_list_response import TypeListResponse as TypeListResponse
from .type_update_params import TypeUpdateParams as TypeUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BillingModel"]


class BillingModel(BaseModel):
    billing_model_id: Optional[str] = None

    created_on: datetime

    default: bool

    name: str

    type: Literal["costplus"]

    updated_on: datetime

    default_price_modifier: Optional[float] = None

    prepaid_amount: Optional[float] = None

    prepaid_max: Optional[float] = None

    threshold: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from payi.types import (
    BudgetResponse,
    DefaultResponse,
    PagedBudgetList,
    BudgetHistoryResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBudgets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Payi) -> None:
        budget = client.budgets.create(
            budget_name="x",
            max=1,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Payi) -> None:
        budget = client.budgets.create(
            budget_name="x",
            max=1,
            base_cost_estimate="max",
            billing_model_id="billing_model_id",
            budget_response_type="block",
            budget_tags=["tag1", "tag2"],
            cost_basis="base",
            currency="usd",
            threshold=0,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.create(
            budget_name="x",
            max=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.create(
            budget_name="x",
            max=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Payi) -> None:
        budget = client.budgets.retrieve(
            "budget_id",
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.retrieve(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.retrieve(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Payi) -> None:
        budget = client.budgets.update(
            budget_id="budget_id",
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Payi) -> None:
        budget = client.budgets.update(
            budget_id="budget_id",
            budget_name="budget_name",
            max=1,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.update(
            budget_id="budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.update(
            budget_id="budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.with_raw_response.update(
                budget_id="",
            )

    @parametrize
    def test_method_list(self, client: Payi) -> None:
        budget = client.budgets.list()
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Payi) -> None:
        budget = client.budgets.list(
            budget_name="budget_name",
            page_number=0,
            page_size=0,
            sort_ascending=True,
            sort_by="sort_by",
            tags="tags",
        )
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(PagedBudgetList, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Payi) -> None:
        budget = client.budgets.delete(
            "budget_id",
        )
        assert_matches_type(DefaultResponse, budget, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.delete(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(DefaultResponse, budget, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.delete(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(DefaultResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_reset(self, client: Payi) -> None:
        budget = client.budgets.reset(
            "budget_id",
        )
        assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

    @parametrize
    def test_raw_response_reset(self, client: Payi) -> None:
        response = client.budgets.with_raw_response.reset(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = response.parse()
        assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

    @parametrize
    def test_streaming_response_reset(self, client: Payi) -> None:
        with client.budgets.with_streaming_response.reset(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = response.parse()
            assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_reset(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            client.budgets.with_raw_response.reset(
                "",
            )


class TestAsyncBudgets:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.create(
            budget_name="x",
            max=1,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.create(
            budget_name="x",
            max=1,
            base_cost_estimate="max",
            billing_model_id="billing_model_id",
            budget_response_type="block",
            budget_tags=["tag1", "tag2"],
            cost_basis="base",
            currency="usd",
            threshold=0,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.create(
            budget_name="x",
            max=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.create(
            budget_name="x",
            max=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.retrieve(
            "budget_id",
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.retrieve(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.retrieve(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.update(
            budget_id="budget_id",
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.update(
            budget_id="budget_id",
            budget_name="budget_name",
            max=1,
        )
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.update(
            budget_id="budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(BudgetResponse, budget, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.update(
            budget_id="budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(BudgetResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.with_raw_response.update(
                budget_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.list()
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.list(
            budget_name="budget_name",
            page_number=0,
            page_size=0,
            sort_ascending=True,
            sort_by="sort_by",
            tags="tags",
        )
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(PagedBudgetList, budget, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(PagedBudgetList, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.delete(
            "budget_id",
        )
        assert_matches_type(DefaultResponse, budget, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.delete(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(DefaultResponse, budget, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.delete(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(DefaultResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_reset(self, async_client: AsyncPayi) -> None:
        budget = await async_client.budgets.reset(
            "budget_id",
        )
        assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

    @parametrize
    async def test_raw_response_reset(self, async_client: AsyncPayi) -> None:
        response = await async_client.budgets.with_raw_response.reset(
            "budget_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        budget = await response.parse()
        assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

    @parametrize
    async def test_streaming_response_reset(self, async_client: AsyncPayi) -> None:
        async with async_client.budgets.with_streaming_response.reset(
            "budget_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            budget = await response.parse()
            assert_matches_type(BudgetHistoryResponse, budget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_reset(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `budget_id` but received ''"):
            await async_client.budgets.with_raw_response.reset(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IngestUnitsParams", "Units"]


class IngestUnitsParams(TypedDict, total=False):
    category: Required[str]

    resource: Required[str]

    units: Required[Dict[str, Units]]

    end_to_end_latency_ms: Optional[int]

    event_timestamp: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    http_status_code: Optional[int]

    provisioned_resource_name: Optional[str]

    time_to_first_token_ms: Optional[int]

    budget_ids: Annotated[Union[list[str], None], PropertyInfo(alias="xProxy-Budget-IDs")]

    request_tags: Annotated[Union[list[str], None], PropertyInfo(alias="xProxy-Request-Tags")]

    experience_name: Annotated[Union[str, None], PropertyInfo(alias="xProxy-Experience-Name")]

    experience_id: Annotated[Union[str, None], PropertyInfo(alias="xProxy-Experience-Id")]

    user_id: Annotated[Union[str, None], PropertyInfo(alias="xProxy-User-ID")]

class Units(TypedDict, total=False):
    input: int

    output: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["CsatCreateParams"]


class CsatCreateParams(TypedDict, total=False):
    csat_rating: Required[int]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BillingModelUpdateParams"]


class BillingModelUpdateParams(TypedDict, total=False):
    type: Required[Literal["costplus"]]

    default_price_modifier: Optional[float]

    name: Optional[str]

    prepaid_amount: Optional[float]

    prepaid_max: Optional[float]

    threshold: Optional[float]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DefaultResponse"]


class DefaultResponse(BaseModel):
    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CostDetails"]


class CostDetails(BaseModel):
    base: float

    overage_base: Optional[float] = None

    overrun_base: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .types import (
    TypesResource,
    AsyncTypesResource,
    TypesResourceWithRawResponse,
    AsyncTypesResourceWithRawResponse,
    TypesResourceWithStreamingResponse,
    AsyncTypesResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.experience_instance import ExperienceInstance

__all__ = ["ExperiencesResource", "AsyncExperiencesResource"]


class ExperiencesResource(SyncAPIResource):
    @cached_property
    def types(self) -> TypesResource:
        return TypesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ExperiencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return ExperiencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExperiencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return ExperiencesResourceWithStreamingResponse(self)

    def create(
        self,
        experience_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Create an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_name:
            raise ValueError(f"Expected a non-empty value for `experience_name` but received {experience_name!r}")
        return self._post(
            f"/api/v1/experiences/instances/{experience_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )

    def retrieve(
        self,
        experience_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Get an Experience details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return self._get(
            f"/api/v1/experiences/instances/{experience_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )

    def delete(
        self,
        experience_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Delete an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return self._delete(
            f"/api/v1/experiences/instances/{experience_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )


class AsyncExperiencesResource(AsyncAPIResource):
    @cached_property
    def types(self) -> AsyncTypesResource:
        return AsyncTypesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncExperiencesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExperiencesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExperiencesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return AsyncExperiencesResourceWithStreamingResponse(self)

    async def create(
        self,
        experience_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Create an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_name:
            raise ValueError(f"Expected a non-empty value for `experience_name` but received {experience_name!r}")
        return await self._post(
            f"/api/v1/experiences/instances/{experience_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )

    async def retrieve(
        self,
        experience_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Get an Experience details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return await self._get(
            f"/api/v1/experiences/instances/{experience_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )

    async def delete(
        self,
        experience_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExperienceInstance:
        """
        Delete an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return await self._delete(
            f"/api/v1/experiences/instances/{experience_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExperienceInstance,
        )


class ExperiencesResourceWithRawResponse:
    def __init__(self, experiences: ExperiencesResource) -> None:
        self._experiences = experiences

        self.create = to_raw_response_wrapper(
            experiences.create,
        )
        self.retrieve = to_raw_response_wrapper(
            experiences.retrieve,
        )
        self.delete = to_raw_response_wrapper(
            experiences.delete,
        )

    @cached_property
    def types(self) -> TypesResourceWithRawResponse:
        return TypesResourceWithRawResponse(self._experiences.types)


class AsyncExperiencesResourceWithRawResponse:
    def __init__(self, experiences: AsyncExperiencesResource) -> None:
        self._experiences = experiences

        self.create = async_to_raw_response_wrapper(
            experiences.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            experiences.retrieve,
        )
        self.delete = async_to_raw_response_wrapper(
            experiences.delete,
        )

    @cached_property
    def types(self) -> AsyncTypesResourceWithRawResponse:
        return AsyncTypesResourceWithRawResponse(self._experiences.types)


class ExperiencesResourceWithStreamingResponse:
    def __init__(self, experiences: ExperiencesResource) -> None:
        self._experiences = experiences

        self.create = to_streamed_response_wrapper(
            experiences.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            experiences.retrieve,
        )
        self.delete = to_streamed_response_wrapper(
            experiences.delete,
        )

    @cached_property
    def types(self) -> TypesResourceWithStreamingResponse:
        return TypesResourceWithStreamingResponse(self._experiences.types)


class AsyncExperiencesResourceWithStreamingResponse:
    def __init__(self, experiences: AsyncExperiencesResource) -> None:
        self._experiences = experiences

        self.create = async_to_streamed_response_wrapper(
            experiences.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            experiences.retrieve,
        )
        self.delete = async_to_streamed_response_wrapper(
            experiences.delete,
        )

    @cached_property
    def types(self) -> AsyncTypesResourceWithStreamingResponse:
        return AsyncTypesResourceWithStreamingResponse(self._experiences.types)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BudgetCreateParams"]


class BudgetCreateParams(TypedDict, total=False):
    budget_name: Required[str]

    max: Required[float]

    base_cost_estimate: Literal["max"]

    billing_model_id: Optional[str]

    budget_response_type: Literal["block", "allow"]

    budget_tags: Optional[List[str]]

    cost_basis: Literal["base", "billed"]

    currency: Literal["usd"]

    threshold: Optional[float]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ExperienceType"]


class ExperienceType(BaseModel):
    description: str

    name: str

    request_id: str

    logging_enabled: Optional[bool] = None



# Step 1: Define the new methods outside of the Payi class definition
from typing import Dict, List, Union


def create_budget_header_from_ids(budget_ids: List[str]) -> Dict[str, str]:
    if not isinstance(budget_ids, list): # type: ignore
        raise TypeError("budget_ids must be a list")

    valid_ids = [id.strip() for id in budget_ids if isinstance(id, str) and id.strip()] # type: ignore

    return {"xProxy-Budget-IDs": ",".join(valid_ids)} if valid_ids else {}

def create_request_header_from_tags(request_tags: List[str]) -> Dict[str, str]:
    if not isinstance(request_tags, list): # type: ignore
        raise TypeError("request_tags must be a list")

    valid_tags = [tag.strip() for tag in request_tags if isinstance(tag, str) and tag.strip()] # type: ignore
    
    return {"xProxy-Request-Tags": ",".join(valid_tags)} if valid_tags else {}

def create_headers(
    budget_ids: Union[List[str], None] = None, 
    request_tags: Union[List[str], None] = None,
    user_id: Union[str, None] = None,
    experience_id: Union[str, None] = None, 
    ) -> Dict[str, str]:
    headers: Dict[str, str] = {}

    if budget_ids:
        headers.update(create_budget_header_from_ids(budget_ids))
    if request_tags:
        headers.update(create_request_header_from_tags(request_tags))
    if user_id:
        headers.update({"xProxy-User-ID": user_id})
    if experience_id:
        headers.update({"xProxy-Experience-Id": experience_id})

    return headers


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from payi import Payi, AsyncPayi
from payi.types import (
    PriceModifier,
    PriceModifierRetrieveResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPriceModifiers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Payi) -> None:
        price_modifier = client.price_modifiers.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Payi) -> None:
        response = client.price_modifiers.with_raw_response.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = response.parse()
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Payi) -> None:
        with client.price_modifiers.with_streaming_response.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = response.parse()
            assert_matches_type(PriceModifier, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Payi) -> None:
        price_modifier = client.price_modifiers.retrieve(
            "billing_model_id",
        )
        assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Payi) -> None:
        response = client.price_modifiers.with_raw_response.retrieve(
            "billing_model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = response.parse()
        assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Payi) -> None:
        with client.price_modifiers.with_streaming_response.retrieve(
            "billing_model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = response.parse()
            assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Payi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `billing_model_id` but received ''"):
            client.price_modifiers.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Payi) -> None:
        price_modifier = client.price_modifiers.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Payi) -> None:
        response = client.price_modifiers.with_raw_response.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = response.parse()
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Payi) -> None:
        with client.price_modifiers.with_streaming_response.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = response.parse()
            assert_matches_type(PriceModifier, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPriceModifiers:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncPayi) -> None:
        price_modifier = await async_client.price_modifiers.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPayi) -> None:
        response = await async_client.price_modifiers.with_raw_response.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = await response.parse()
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPayi) -> None:
        async with async_client.price_modifiers.with_streaming_response.create(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = await response.parse()
            assert_matches_type(PriceModifier, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncPayi) -> None:
        price_modifier = await async_client.price_modifiers.retrieve(
            "billing_model_id",
        )
        assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncPayi) -> None:
        response = await async_client.price_modifiers.with_raw_response.retrieve(
            "billing_model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = await response.parse()
        assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncPayi) -> None:
        async with async_client.price_modifiers.with_streaming_response.retrieve(
            "billing_model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = await response.parse()
            assert_matches_type(PriceModifierRetrieveResponse, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncPayi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `billing_model_id` but received ''"):
            await async_client.price_modifiers.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncPayi) -> None:
        price_modifier = await async_client.price_modifiers.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPayi) -> None:
        response = await async_client.price_modifiers.with_raw_response.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        price_modifier = await response.parse()
        assert_matches_type(PriceModifier, price_modifier, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPayi) -> None:
        async with async_client.price_modifiers.with_streaming_response.update(
            billing_model_id="x",
            category="x",
            price_modifier=0,
            resource="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            price_modifier = await response.parse()
            assert_matches_type(PriceModifier, price_modifier, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IngestEventParam", "Units"]


class Units(TypedDict, total=False):
    input: int

    output: int


class IngestEventParam(TypedDict, total=False):
    category: Required[str]

    resource: Required[str]

    units: Required[Dict[str, Units]]

    budget_ids: Optional[List[str]]

    csat_rating: Optional[int]

    end_to_end_latency_ms: Optional[int]

    event_timestamp: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    experience_id: Optional[str]

    experience_name: Optional[str]

    http_status_code: Optional[int]

    provisioned_resource_name: Optional[str]

    request_tags: Optional[List[str]]

    time_to_first_token_ms: Optional[int]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .total_cost_data import TotalCostData

__all__ = ["BudgetResponse", "Budget"]


class Budget(BaseModel):
    base_cost_estimate: Literal["max"]

    budget_creation_timestamp: datetime

    budget_id: str

    budget_name: str

    budget_response_type: Literal["block", "allow"]

    budget_update_timestamp: datetime

    currency: Literal["usd"]

    max: float

    totals: TotalCostData

    budget_tags: Optional[List[str]] = None

    threshold: Optional[float] = None


class BudgetResponse(BaseModel):
    budget: Budget

    request_id: str

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel
from .cost_details import CostDetails

__all__ = ["CostData"]


class CostData(BaseModel):
    input: CostDetails

    output: CostDetails

    total: CostDetails



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import csat_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.csat import Csat
from .._base_client import make_request_options

__all__ = ["CsatResource", "AsyncCsatResource"]


class CsatResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CsatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return CsatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CsatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return CsatResourceWithStreamingResponse(self)

    def create(
        self,
        experience_id: str,
        *,
        csat_rating: int,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Csat:
        """
        Created a CSAT rating for an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return self._post(
            f"/api/v1/csat/experiences/{experience_id}",
            body=maybe_transform(
                {
                    "csat_rating": csat_rating,
                    "user_id": user_id,
                },
                csat_create_params.CsatCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Csat,
        )


class AsyncCsatResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCsatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Pay-i/pay-i-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCsatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCsatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Pay-i/pay-i-python#with_streaming_response
        """
        return AsyncCsatResourceWithStreamingResponse(self)

    async def create(
        self,
        experience_id: str,
        *,
        csat_rating: int,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Csat:
        """
        Created a CSAT rating for an Experience

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not experience_id:
            raise ValueError(f"Expected a non-empty value for `experience_id` but received {experience_id!r}")
        return await self._post(
            f"/api/v1/csat/experiences/{experience_id}",
            body=await async_maybe_transform(
                {
                    "csat_rating": csat_rating,
                    "user_id": user_id,
                },
                csat_create_params.CsatCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Csat,
        )


class CsatResourceWithRawResponse:
    def __init__(self, csat: CsatResource) -> None:
        self._csat = csat

        self.create = to_raw_response_wrapper(
            csat.create,
        )


class AsyncCsatResourceWithRawResponse:
    def __init__(self, csat: AsyncCsatResource) -> None:
        self._csat = csat

        self.create = async_to_raw_response_wrapper(
            csat.create,
        )


class CsatResourceWithStreamingResponse:
    def __init__(self, csat: CsatResource) -> None:
        self._csat = csat

        self.create = to_streamed_response_wrapper(
            csat.create,
        )


class AsyncCsatResourceWithStreamingResponse:
    def __init__(self, csat: AsyncCsatResource) -> None:
        self._csat = csat

        self.create = async_to_streamed_response_wrapper(
            csat.create,
        )
